@@ -1,4 +1,4 @@
-/* $Id: cmd-pipe-pane.c,v 1.13 2010/06/15 20:25:40 tcunha Exp $ */
+/* $Id: cmd-pipe-pane.c,v 1.15 2010/10/24 00:45:57 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -96,7 +96,7 @@ cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 	case 0:
 		/* Child process. */
 		close(pipe_fd[0]);
-		clear_signals();
+		clear_signals(1);
 
 		if (dup2(pipe_fd[1], STDIN_FILENO) == -1)
 			_exit(1);
@@ -111,6 +111,8 @@ cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 		if (null_fd != STDOUT_FILENO && null_fd != STDERR_FILENO)
 			close(null_fd);
 
+		closefrom(STDERR_FILENO + 1);
+
 		command = status_replace(c, NULL, data->arg, time(NULL), 0);
 		execl(_PATH_BSHELL, "sh", "-c", command, (char *) NULL);
 		_exit(1);
@@ -129,8 +131,6 @@ cmd_pipe_pane_exec(struct cmd *self, struct cmd_ctx *ctx)
 			fatal("fcntl failed");
 		if (fcntl(wp->pipe_fd, F_SETFL, mode|O_NONBLOCK) == -1)
 			fatal("fcntl failed");
-		if (fcntl(wp->pipe_fd, F_SETFD, FD_CLOEXEC) == -1)
-			fatal("fcntl failed");
 		return (0);
 	}
 }
