@@ -1,4 +1,4 @@
-/* $Id: compat.h,v 1.25 2010/06/06 13:00:46 tcunha Exp $ */
+/* $Id: compat.h,v 1.31 2010/11/11 20:45:49 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -16,6 +16,20 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#ifndef COMPAT_H
+#define COMPAT_H
+
+#ifndef __GNUC__
+#define __attribute__(a)
+#endif
+
+#ifndef __dead
+#define __dead __attribute__ ((__noreturn__))
+#endif
+#ifndef __packed
+#define __packed __attribute__ ((__packed__))
+#endif
+
 #ifndef HAVE_U_INT
 typedef uint8_t u_int8_t;
 typedef uint16_t u_int16_t;
@@ -49,14 +63,6 @@ typedef uint64_t u_int64_t;
 #include "compat/bitstring.h"
 #endif
 
-#ifdef HAVE_GETOPT
-#include <getopt.h>
-#endif
-
-#ifdef HAVE_CRYPT_H
-#include <crypt.h>
-#endif
-
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif
@@ -79,10 +85,16 @@ typedef uint64_t u_int64_t;
 #include "compat/vis.h"
 #endif
 
-#ifndef HAVE_IMSG
-#include "compat/imsg.h"
-#else
+#ifdef HAVE_IMSG
 #include <imsg.h>
+#else
+#include "compat/imsg.h"
+#endif
+
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#else
+#include <inttypes.h>
 #endif
 
 #ifdef HAVE_BROKEN_CMSG_FIRSTHDR
@@ -123,13 +135,6 @@ typedef uint64_t u_int64_t;
 #define SUN_LEN(sun) (sizeof (sun)->sun_path)
 #endif
 
-#ifndef __dead
-#define __dead __attribute__ ((__noreturn__))
-#endif
-#ifndef __packed
-#define __packed __attribute__ ((__packed__))
-#endif
-
 #ifndef timercmp
 #define	timercmp(tvp, uvp, cmp)						\
 	(((tvp)->tv_sec == (uvp)->tv_sec) ?				\
@@ -154,9 +159,18 @@ typedef uint64_t u_int64_t;
 #endif
 
 #ifndef HAVE_BZERO
+#undef bzero
 #define bzero(buf, len) memset(buf, 0, len);
 #endif
 
+#ifndef HAVE_CLOSEFROM
+/* closefrom.c */
+#define HAVE_FCNTL_H
+#define HAVE_DIRENT_H
+#define HAVE_SYSCONF
+void	closefrom(int);
+#endif
+
 #ifndef HAVE_STRCASESTR
 /* strcasestr.c */
 char		*strcasestr(const char *, const char *);
@@ -189,6 +203,7 @@ int	 	 daemon(int, int);
 
 #ifndef HAVE_FORKPTY
 /* forkpty.c */
+#include <sys/ioctl.h>
 pid_t		 forkpty(int *, char *, struct termios *, struct winsize *);
 #endif
 
@@ -209,7 +224,9 @@ int		 setenv(const char *, const char *, int);
 int		 unsetenv(const char *);
 #endif
 
-#ifndef HAVE_GETOPT
+#ifdef HAVE_GETOPT
+#include <getopt.h>
+#else
 /* getopt.c */
 extern int	BSDopterr;
 extern int	BSDoptind;
@@ -224,3 +241,5 @@ int	BSDgetopt(int, char *const *, const char *);
 #define optreset           BSDoptreset
 #define optarg             BSDoptarg
 #endif
+
+#endif /* COMPAT_H */
