@@ -1,4 +1,4 @@
-/* $Id: job.c,v 1.17 2010/05/14 14:30:01 tcunha Exp $ */
+/* $Id: job.c,v 1.19 2010/10/24 00:45:57 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -148,7 +148,7 @@ job_run(struct job *job)
 	case -1:
 		return (-1);
 	case 0:		/* child */
-		clear_signals();
+		clear_signals(1);
 
 		environ_push(&global_environ);
 
@@ -168,6 +168,8 @@ job_run(struct job *job)
 		if (nullfd != STDIN_FILENO && nullfd != STDERR_FILENO)
 			close(nullfd);
 
+		closefrom(STDERR_FILENO + 1);
+
 		execl(_PATH_BSHELL, "sh", "-c", job->cmd, (char *) NULL);
 		fatal("execl failed");
 	default:	/* parent */
@@ -178,8 +180,6 @@ job_run(struct job *job)
 			fatal("fcntl failed");
 		if (fcntl(job->fd, F_SETFL, mode|O_NONBLOCK) == -1)
 			fatal("fcntl failed");
-		if (fcntl(job->fd, F_SETFD, FD_CLOEXEC) == -1)
-			fatal("fcntl failed");
 
 		if (job->event != NULL)
 			bufferevent_free(job->event);
