@@ -1,4 +1,4 @@
-/* $Id: key-bindings.c,v 1.94 2010/07/02 02:43:01 tcunha Exp $ */
+/* $Id: key-bindings.c,v 1.97 2010/12/11 18:42:20 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -108,6 +108,8 @@ key_bindings_init(void)
 		{ '#', 			  0, &cmd_list_buffers_entry },
 		{ '%', 			  0, &cmd_split_window_entry },
 		{ '&', 			  0, &cmd_confirm_before_entry },
+		{ '(',                    0, &cmd_switch_client_entry },
+		{ ')',                    0, &cmd_switch_client_entry },
 		{ ',', 			  0, &cmd_command_prompt_entry },
 		{ '-', 			  0, &cmd_delete_buffer_entry },
 		{ '.', 			  0, &cmd_command_prompt_entry },
@@ -122,9 +124,11 @@ key_bindings_init(void)
 		{ '8', 			  0, &cmd_select_window_entry },
 		{ '9', 			  0, &cmd_select_window_entry },
 		{ ':', 			  0, &cmd_command_prompt_entry },
+		{ ';', 			  0, &cmd_last_pane_entry },
 		{ '=', 			  0, &cmd_choose_buffer_entry },
 		{ '?', 			  0, &cmd_list_keys_entry },
 		{ 'D',			  0, &cmd_choose_client_entry },
+		{ 'L',			  0, &cmd_switch_client_entry },
 		{ '[', 			  0, &cmd_copy_mode_entry },
 		{ '\'',			  0, &cmd_command_prompt_entry },
 		{ '\002', /* C-b */	  0, &cmd_send_prefix_entry },
