@@ -1,4 +1,4 @@
-/* $Id: server-client.c,v 1.35 2010/07/17 14:36:40 tcunha Exp $ */
+/* $Id: server-client.c,v 1.48 2010/12/22 15:31:00 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,9 +28,14 @@
 
 void	server_client_handle_key(int, struct mouse_event *, void *);
 void	server_client_repeat_timer(int, short, void *);
+void	server_client_check_exit(struct client *);
+void	server_client_check_backoff(struct client *);
 void	server_client_check_redraw(struct client *);
 void	server_client_set_title(struct client *);
 void	server_client_reset_state(struct client *);
+void	server_client_in_callback(struct bufferevent *, short, void *);
+void	server_client_out_callback(struct bufferevent *, short, void *);
+void	server_client_err_callback(struct bufferevent *, short, void *);
 
 int	server_client_msg_dispatch(struct client *);
 void	server_client_msg_command(struct client *, struct msg_command_data *);
@@ -54,8 +59,6 @@ server_client_create(int fd)
 		fatal("fcntl failed");
 	if (fcntl(fd, F_SETFL, mode|O_NONBLOCK) == -1)
 		fatal("fcntl failed");
-	if (fcntl(fd, F_SETFD, FD_CLOEXEC) == -1)
-		fatal("fcntl failed");
 
 	c = xcalloc(1, sizeof *c);
 	c->references = 0;
@@ -66,16 +69,15 @@ server_client_create(int fd)
 		fatal("gettimeofday failed");
 	memcpy(&c->activity_time, &c->creation_time, sizeof c->activity_time);
 
-	ARRAY_INIT(&c->prompt_hdata);
-
-	c->stdin_file = NULL;
-	c->stdout_file = NULL;
-	c->stderr_file = NULL;
+	c->stdin_event = NULL;
+	c->stdout_event = NULL;
+	c->stderr_event = NULL;
 
 	c->tty.fd = -1;
 	c->title = NULL;
 
 	c->session = NULL;
+	c->last_session = NULL;
 	c->tty.sx = 80;
 	c->tty.sy = 24;
 
@@ -121,12 +123,18 @@ server_client_lost(struct client *c)
 	if (c->flags & CLIENT_TERMINAL)
 		tty_free(&c->tty);
 
-	if (c->stdin_file != NULL)
-		fclose(c->stdin_file);
-	if (c->stdout_file != NULL)
-		fclose(c->stdout_file);
-	if (c->stderr_file != NULL)
-		fclose(c->stderr_file);
+	if (c->stdin_fd != -1)
+		close(c->stdin_fd);
+	if (c->stdin_event != NULL)
+		bufferevent_free(c->stdin_event);
+	if (c->stdout_fd != -1)
+		close(c->stdout_fd);
+	if (c->stdout_event != NULL)
+		bufferevent_free(c->stdout_event);
+	if (c->stderr_fd != -1)
+		close(c->stderr_fd);
+	if (c->stderr_event != NULL)
+		bufferevent_free(c->stderr_event);
 
 	screen_free(&c->status);
 	job_tree_free(&c->status_jobs);
@@ -151,9 +159,6 @@ server_client_lost(struct client *c)
 		xfree(c->prompt_string);
 	if (c->prompt_buffer != NULL)
 		xfree(c->prompt_buffer);
-	for (i = 0; i < ARRAY_LENGTH(&c->prompt_hdata); i++)
-		xfree(ARRAY_ITEM(&c->prompt_hdata, i));
-	ARRAY_FREE(&c->prompt_hdata);
 
 	if (c->cwd != NULL)
 		xfree(c->cwd);
@@ -173,6 +178,7 @@ server_client_lost(struct client *c)
 	c->flags |= CLIENT_DEAD;
 
 	recalculate_sizes();
+	server_check_unattached();
 	server_update_socket();
 }
 
@@ -389,11 +395,14 @@ server_client_loop(void)
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
-		if (c == NULL || c->session == NULL)
+		if (c == NULL)
 			continue;
 
-		server_client_check_redraw(c);
-		server_client_reset_state(c);
+		server_client_check_exit(c);
+		if (c->session != NULL) {
+			server_client_check_redraw(c);
+			server_client_reset_state(c);
+		}
 	}
 
 	/*
@@ -456,6 +465,72 @@ server_client_repeat_timer(unused int fd, unused short events, void *data)
 		c->flags &= ~(CLIENT_PREFIX|CLIENT_REPEAT);
 }
 
+/* Check if client should be exited. */
+void
+server_client_check_exit(struct client *c)
+{
+	struct msg_exit_data	exitdata;
+
+	if (!(c->flags & CLIENT_EXIT))
+		return;
+
+	if (c->stdout_fd != -1 && c->stdout_event != NULL &&
+	    EVBUFFER_LENGTH(c->stdout_event->output) != 0)
+		return;
+	if (c->stderr_fd != -1 && c->stderr_event != NULL &&
+	    EVBUFFER_LENGTH(c->stderr_event->output) != 0)
+		return;
+
+	exitdata.retcode = c->retcode;
+	server_write_client(c, MSG_EXIT, &exitdata, sizeof exitdata);
+
+	c->flags &= ~CLIENT_EXIT;
+}
+
+/*
+ * Check if the client should backoff. During backoff, data from external
+ * programs is not written to the terminal. When the existing data drains, the
+ * client is redrawn.
+ *
+ * There are two backoff phases - both the tty and client have backoff flags -
+ * the first to allow existing data to drain and the latter to ensure backoff
+ * is disabled until the redraw has finished and prevent the redraw triggering
+ * another backoff.
+ */
+void
+server_client_check_backoff(struct client *c)
+{
+	struct tty	*tty = &c->tty;
+	size_t		 used;
+
+	used = EVBUFFER_LENGTH(tty->event->output);
+
+	/*
+	 * If in the second backoff phase (redrawing), don't check backoff
+	 * until the redraw has completed (or enough of it to drop below the
+	 * backoff threshold).
+	 */
+	if (c->flags & CLIENT_BACKOFF) {
+		if (used > BACKOFF_THRESHOLD)
+			return;
+		c->flags &= ~CLIENT_BACKOFF;
+		return;
+	}
+
+	/* Once drained, allow data through again and schedule redraw. */
+	if (tty->flags & TTY_BACKOFF) {
+		if (used != 0)
+			return;
+		tty->flags &= ~TTY_BACKOFF;
+		c->flags |= (CLIENT_BACKOFF|CLIENT_REDRAWWINDOW|CLIENT_STATUS);
+		return;
+	}
+
+	/* If too much data, start backoff. */
+	if (used > BACKOFF_THRESHOLD)
+		tty->flags |= TTY_BACKOFF;
+}
+
 /* Check for client redraws. */
 void
 server_client_check_redraw(struct client *c)
@@ -484,6 +559,10 @@ server_client_check_redraw(struct client *c)
 	if (c->flags & CLIENT_REDRAW) {
 		screen_redraw_screen(c, 0, 0);
 		c->flags &= ~(CLIENT_STATUS|CLIENT_BORDERS);
+	} else if (c->flags & CLIENT_REDRAWWINDOW) {
+		TAILQ_FOREACH(wp, &c->session->curw->window->panes, entry)
+			screen_redraw_pane(c, wp);
+		c->flags &= ~CLIENT_REDRAWWINDOW;
 	} else {
 		TAILQ_FOREACH(wp, &c->session->curw->window->panes, entry) {
 			if (wp->flags & PANE_REDRAW)
@@ -522,6 +601,52 @@ server_client_set_title(struct client *c)
 	xfree(title);
 }
 
+/*
+ * Error callback for client stdin. Caller must increase reference count when
+ * enabling event!
+ */
+void
+server_client_in_callback(
+    unused struct bufferevent *bufev, unused short what, void *data)
+{
+	struct client	*c = data;
+
+	c->references--;
+	if (c->flags & CLIENT_DEAD)
+		return;
+
+	bufferevent_disable(c->stdin_event, EV_READ|EV_WRITE);
+	close(c->stdin_fd);
+	c->stdin_fd = -1;
+
+	if (c->stdin_callback != NULL)
+		c->stdin_callback(c, c->stdin_data);
+}
+
+/* Error callback for client stdout. */
+void
+server_client_out_callback(
+    unused struct bufferevent *bufev, unused short what, unused void *data)
+{
+	struct client	*c = data;
+
+	bufferevent_disable(c->stdout_event, EV_READ|EV_WRITE);
+	close(c->stdout_fd);
+	c->stdout_fd = -1;
+}
+
+/* Error callback for client stderr. */
+void
+server_client_err_callback(
+    unused struct bufferevent *bufev, unused short what, unused void *data)
+{
+	struct client	*c = data;
+
+	bufferevent_disable(c->stderr_event, EV_READ|EV_WRITE);
+	close(c->stderr_fd);
+	c->stderr_fd = -1;
+}
+
 /* Dispatch message from client. */
 int
 server_client_msg_dispatch(struct client *c)
@@ -531,6 +656,7 @@ server_client_msg_dispatch(struct client *c)
 	struct msg_identify_data identifydata;
 	struct msg_environ_data	 environdata;
 	ssize_t			 n, datalen;
+	int			 mode;
 
 	if ((n = imsg_read(&c->ibuf)) == -1 || n == 0)
 		return (-1);
@@ -565,9 +691,15 @@ server_client_msg_dispatch(struct client *c)
 				fatalx("MSG_IDENTIFY missing fd");
 			memcpy(&identifydata, imsg.data, sizeof identifydata);
 
-			c->stdin_file = fdopen(imsg.fd, "r");
-			if (c->stdin_file == NULL)
-				fatal("fdopen(stdin) failed");
+			c->stdin_fd = imsg.fd;
+			c->stdin_event = bufferevent_new(c->stdin_fd,
+			    NULL, NULL, server_client_in_callback, c);
+			if (c->stdin_event == NULL)
+				fatalx("failed to create stdin event");
+
+			if ((mode = fcntl(c->stdin_fd, F_GETFL)) != -1)
+				fcntl(c->stdin_fd, F_SETFL, mode|O_NONBLOCK);
+
 			server_client_msg_identify(c, &identifydata, imsg.fd);
 			break;
 		case MSG_STDOUT:
@@ -576,9 +708,14 @@ server_client_msg_dispatch(struct client *c)
 			if (imsg.fd == -1)
 				fatalx("MSG_STDOUT missing fd");
 
-			c->stdout_file = fdopen(imsg.fd, "w");
-			if (c->stdout_file == NULL)
-				fatal("fdopen(stdout) failed");
+			c->stdout_fd = imsg.fd;
+			c->stdout_event = bufferevent_new(c->stdout_fd,
+			    NULL, NULL, server_client_out_callback, c);
+			if (c->stdout_event == NULL)
+				fatalx("failed to create stdout event");
+
+			if ((mode = fcntl(c->stdout_fd, F_GETFL)) != -1)
+				fcntl(c->stdout_fd, F_SETFL, mode|O_NONBLOCK);
 			break;
 		case MSG_STDERR:
 			if (datalen != 0)
@@ -586,9 +723,14 @@ server_client_msg_dispatch(struct client *c)
 			if (imsg.fd == -1)
 				fatalx("MSG_STDERR missing fd");
 
-			c->stderr_file = fdopen(imsg.fd, "w");
-			if (c->stderr_file == NULL)
-				fatal("fdopen(stderr) failed");
+			c->stderr_fd = imsg.fd;
+			c->stderr_event = bufferevent_new(c->stderr_fd,
+			    NULL, NULL, server_client_err_callback, c);
+			if (c->stderr_event == NULL)
+				fatalx("failed to create stderr event");
+
+			if ((mode = fcntl(c->stderr_fd, F_GETFL)) != -1)
+				fcntl(c->stderr_fd, F_SETFL, mode|O_NONBLOCK);
 			break;
 		case MSG_RESIZE:
 			if (datalen != 0)
@@ -658,12 +800,10 @@ server_client_msg_error(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_list	ap;
 
 	va_start(ap, fmt);
-	vfprintf(ctx->cmdclient->stderr_file, fmt, ap);
+	evbuffer_add_vprintf(ctx->cmdclient->stderr_event->output, fmt, ap);
 	va_end(ap);
 
-	fputc('\n', ctx->cmdclient->stderr_file);
-	fflush(ctx->cmdclient->stderr_file);
-
+	bufferevent_write(ctx->cmdclient->stderr_event, "\n", 1);
 	ctx->cmdclient->retcode = 1;
 }
 
@@ -674,11 +814,10 @@ server_client_msg_print(struct cmd_ctx *ctx, const char *fmt, ...)
 	va_list	ap;
 
 	va_start(ap, fmt);
-	vfprintf(ctx->cmdclient->stdout_file, fmt, ap);
+	evbuffer_add_vprintf(ctx->cmdclient->stdout_event->output, fmt, ap);
 	va_end(ap);
 
-	fputc('\n', ctx->cmdclient->stdout_file);
-	fflush(ctx->cmdclient->stdout_file);
+	bufferevent_write(ctx->cmdclient->stdout_event, "\n", 1);
 }
 
 /* Callback to send print message to client, if not quiet. */
@@ -691,22 +830,20 @@ server_client_msg_info(struct cmd_ctx *ctx, const char *fmt, ...)
 		return;
 
 	va_start(ap, fmt);
-	vfprintf(ctx->cmdclient->stdout_file, fmt, ap);
+	evbuffer_add_vprintf(ctx->cmdclient->stdout_event->output, fmt, ap);
 	va_end(ap);
 
-	fputc('\n', ctx->cmdclient->stderr_file);
-	fflush(ctx->cmdclient->stdout_file);
+	bufferevent_write(ctx->cmdclient->stdout_event, "\n", 1);
 }
 
 /* Handle command message. */
 void
 server_client_msg_command(struct client *c, struct msg_command_data *data)
 {
-	struct cmd_ctx	 	ctx;
-	struct cmd_list	       *cmdlist = NULL;
-	struct msg_exit_data	exitdata;
-	int			argc;
-	char		      **argv, *cause;
+	struct cmd_ctx	 ctx;
+	struct cmd_list	*cmdlist = NULL;
+	int		 argc;
+	char	       **argv, *cause;
 
 	ctx.error = server_client_msg_error;
 	ctx.print = server_client_msg_print;
@@ -737,18 +874,15 @@ server_client_msg_command(struct client *c, struct msg_command_data *data)
 	}
 	cmd_free_argv(argc, argv);
 
-	if (cmd_list_exec(cmdlist, &ctx) != 1) {
-		exitdata.retcode = c->retcode;
-		server_write_client(c, MSG_EXIT, &exitdata, sizeof exitdata);
-	}
+	if (cmd_list_exec(cmdlist, &ctx) != 1)
+		c->flags |= CLIENT_EXIT;
 	cmd_list_free(cmdlist);
 	return;
 
 error:
 	if (cmdlist != NULL)
 		cmd_list_free(cmdlist);
-	exitdata.retcode = c->retcode;
-	server_write_client(c, MSG_EXIT, &exitdata, sizeof exitdata);
+	c->flags |= CLIENT_EXIT;
 }
 
 /* Handle identify message. */
