@@ -1,4 +1,4 @@
-/* $Id: server-window.c,v 1.15 2010/06/22 23:26:18 tcunha Exp $ */
+/* $Id: server-window.c,v 1.19 2010/12/22 15:36:44 tcunha Exp $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -26,34 +26,10 @@
 int	server_window_backoff(struct window_pane *);
 int	server_window_check_bell(struct session *, struct winlink *);
 int	server_window_check_activity(struct session *, struct winlink *);
+int	server_window_check_silence(struct session *, struct winlink *);
 int	server_window_check_content(
 	    struct session *, struct winlink *, struct window_pane *);
 
-/* Check if this window should suspend reading. */
-int
-server_window_backoff(struct window_pane *wp)
-{
-	struct client	*c;
-	u_int		 i;
-
-	if (!window_pane_visible(wp))
-		return (0);
-
-	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
-		c = ARRAY_ITEM(&clients, i);
-		if (c == NULL || c->session == NULL)
-			continue;
-		if ((c->flags & (CLIENT_SUSPENDED|CLIENT_DEAD)) != 0)
-			continue;
-		if (c->session->curw->window != wp->window)
-			continue;
-
-		if (EVBUFFER_LENGTH(c->tty.event->output) > BACKOFF_THRESHOLD)
-			return (1);
-	}
-	return (0);
-}
-
 /* Window functions that need to happen every loop. */
 void
 server_window_loop(void)
@@ -62,34 +38,21 @@ server_window_loop(void)
 	struct winlink		*wl;
 	struct window_pane	*wp;
 	struct session		*s;
-	u_int		 	 i, j;
+	u_int		 	 i;
 
 	for (i = 0; i < ARRAY_LENGTH(&windows); i++) {
 		w = ARRAY_ITEM(&windows, i);
 		if (w == NULL)
 			continue;
 
-		TAILQ_FOREACH(wp, &w->panes, entry) {
-			if (wp->fd == -1)
-				continue;
-			if (!(wp->flags & PANE_FREEZE)) {
-				if (server_window_backoff(wp))
-					bufferevent_disable(wp->event, EV_READ);
-				else
-					bufferevent_enable(wp->event, EV_READ);
-			}
-		}
-
-		for (j = 0; j < ARRAY_LENGTH(&sessions); j++) {
-			s = ARRAY_ITEM(&sessions, j);
-			if (s == NULL)
-				continue;
+		RB_FOREACH(s, sessions, &sessions) {
 			wl = session_has(s, w);
 			if (wl == NULL)
 				continue;
 
 			if (server_window_check_bell(s, wl) ||
-			    server_window_check_activity(s, wl))
+			    server_window_check_activity(s, wl) ||
+			    server_window_check_silence(s, wl))
 				server_status_session(s);
 			TAILQ_FOREACH(wp, &w->panes, entry)
 				server_window_check_content(s, wl, wp);
@@ -109,10 +72,8 @@ server_window_check_bell(struct session *s, struct winlink *wl)
 
 	if (!(w->flags & WINDOW_BELL) || wl->flags & WINLINK_BELL)
 		return (0);
-	if (s->curw == wl)
-		return (0);
-
-	wl->flags |= WINLINK_BELL;
+	if (s->curw != wl)
+		wl->flags |= WINLINK_BELL;
 
 	action = options_get_number(&s->options, "bell-action");
 	switch (action) {
@@ -189,6 +150,55 @@ server_window_check_activity(struct session *s, struct winlink *wl)
 	return (1);
 }
 
+/* Check for silence in window. */
+int
+server_window_check_silence(struct session *s, struct winlink *wl)
+{
+	struct client	*c;
+	struct window	*w = wl->window;
+	struct timeval	 timer;
+	u_int		 i;
+	int		 silence_interval, timer_difference;
+
+	if (!(w->flags & WINDOW_SILENCE) || wl->flags & WINLINK_SILENCE)
+		return (0);
+
+	if (s->curw == wl) {
+		/*
+		 * Reset the timer for this window if we've focused it.  We
+		 * don't want the timer tripping as soon as we've switched away
+		 * from this window.
+		 */
+		if (gettimeofday(&w->silence_timer, NULL) != 0)
+			fatal("gettimeofday failed.");
+
+		return (0);
+	}
+
+	silence_interval = options_get_number(&w->options, "monitor-silence");
+	if (silence_interval == 0)
+		return (0);
+
+	if (gettimeofday(&timer, NULL) != 0)
+		fatal("gettimeofday");
+	timer_difference = timer.tv_sec - w->silence_timer.tv_sec;
+	if (timer_difference <= silence_interval)
+		return (0);
+	wl->flags |= WINLINK_SILENCE;
+
+	if (options_get_number(&s->options, "visual-silence")) {
+		for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
+			c = ARRAY_ITEM(&clients, i);
+			if (c == NULL || c->session != s)
+				continue;
+			status_message_set(c, "Silence in window %u",
+			    winlink_find_by_window(&s->windows, w)->idx);
+		}
+	}
+
+	return (1);
+}
+
 /* Check for content change in window. */
 int
 server_window_check_content(
