@@ -1,4 +1,4 @@
-/* $Id: status.c,v 1.149 2010/06/22 23:26:18 tcunha Exp $ */
+/* $Id: status.c,v 1.151 2010/12/11 16:15:02 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -41,9 +41,14 @@ void	status_replace1(struct client *,
 	    struct winlink *, char **, char **, char *, size_t, int);
 void	status_message_callback(int, short, void *);
 
-void	status_prompt_add_history(struct client *);
+const char *status_prompt_up_history(u_int *);
+const char *status_prompt_down_history(u_int *);
+void	status_prompt_add_history(const char *);
 char   *status_prompt_complete(const char *);
 
+/* Status prompt history. */
+ARRAY_DECL(, char *) status_prompt_history = ARRAY_INITIALIZER;
+
 /* Retrieve options for left string. */
 char *
 status_redraw_get_left(struct client *c,
@@ -395,6 +400,8 @@ status_replace1(struct client *c,struct winlink *wl,
 			tmp[0] = '!';
 		else if (wl->flags & WINLINK_ACTIVITY)
 			tmp[0] = '#';
+		else if (wl->flags & WINLINK_SILENCE)
+			tmp[0] = '~';
 		else if (wl == s->curw)
 			tmp[0] = '*';
 		else if (wl == TAILQ_FIRST(&s->lastw))
@@ -858,6 +865,7 @@ status_prompt_key(struct client *c, int key)
 {
 	struct paste_buffer	*pb;
 	char   			*s, *first, *last, word[64], swapc;
+	const char              *histstr;
 	u_char			 ch;
 	size_t			 size, n, off, idx;
 
@@ -970,29 +978,20 @@ status_prompt_key(struct client *c, int key)
 		}
 		break;
 	case MODEKEYEDIT_HISTORYUP:
-		if (ARRAY_LENGTH(&c->prompt_hdata) == 0)
+		histstr = status_prompt_up_history(&c->prompt_hindex);
+		if (histstr == NULL)
 			break;
 	       	xfree(c->prompt_buffer);
-
-		c->prompt_buffer = xstrdup(ARRAY_ITEM(&c->prompt_hdata,
-		    ARRAY_LENGTH(&c->prompt_hdata) - 1 - c->prompt_hindex));
-		if (c->prompt_hindex != ARRAY_LENGTH(&c->prompt_hdata) - 1)
-			c->prompt_hindex++;
-
+		c->prompt_buffer = xstrdup(histstr);
 		c->prompt_index = strlen(c->prompt_buffer);
 		c->flags |= CLIENT_STATUS;
 		break;
 	case MODEKEYEDIT_HISTORYDOWN:
+		histstr = status_prompt_down_history(&c->prompt_hindex);
+		if (histstr == NULL)
+			break;
 		xfree(c->prompt_buffer);
-
-		if (c->prompt_hindex != 0) {
-			c->prompt_hindex--;
-			c->prompt_buffer = xstrdup(ARRAY_ITEM(
-			    &c->prompt_hdata, ARRAY_LENGTH(
-			    &c->prompt_hdata) - 1 - c->prompt_hindex));
-		} else
-			c->prompt_buffer = xstrdup("");
-
+		c->prompt_buffer = xstrdup(histstr);
 		c->prompt_index = strlen(c->prompt_buffer);
 		c->flags |= CLIENT_STATUS;
 		break;
@@ -1034,7 +1033,7 @@ status_prompt_key(struct client *c, int key)
 		break;
 	case MODEKEYEDIT_ENTER:
 		if (*c->prompt_buffer != '\0')
-			status_prompt_add_history(c);
+			status_prompt_add_history(c->prompt_buffer);
 		if (c->prompt_callbackfn(c->prompt_data, c->prompt_buffer) == 0)
 			status_prompt_clear(c);
 		break;
@@ -1070,20 +1069,56 @@ status_prompt_key(struct client *c, int key)
 	}
 }
 
+/* Get previous line from the history. */
+const char *
+status_prompt_up_history(u_int *idx)
+{
+	u_int size;
+
+	/*
+	 * History runs from 0 to size - 1.
+	 *
+	 * Index is from 0 to size. Zero is empty.
+	 */
+
+	size = ARRAY_LENGTH(&status_prompt_history);
+	if (size == 0 || *idx == size)
+		return (NULL);
+	(*idx)++;
+	return (ARRAY_ITEM(&status_prompt_history, size - *idx));
+}
+
+/* Get next line from the history. */
+const char *
+status_prompt_down_history(u_int *idx)
+{
+	u_int size;
+
+	size = ARRAY_LENGTH(&status_prompt_history);
+	if (size == 0 || *idx == 0)
+		return ("");
+	(*idx)--;
+	if (*idx == 0)
+		return ("");
+	return (ARRAY_ITEM(&status_prompt_history, size - *idx));
+}
+
 /* Add line to the history. */
 void
-status_prompt_add_history(struct client *c)
+status_prompt_add_history(const char *line)
 {
-	if (ARRAY_LENGTH(&c->prompt_hdata) > 0 &&
-	    strcmp(ARRAY_LAST(&c->prompt_hdata), c->prompt_buffer) == 0)
+	u_int size;
+
+	size = ARRAY_LENGTH(&status_prompt_history);
+	if (size > 0 && strcmp(ARRAY_LAST(&status_prompt_history), line) == 0)
 		return;
 
-	if (ARRAY_LENGTH(&c->prompt_hdata) == PROMPT_HISTORY) {
-		xfree(ARRAY_FIRST(&c->prompt_hdata));
-		ARRAY_REMOVE(&c->prompt_hdata, 0);
+	if (size == PROMPT_HISTORY) {
+		xfree(ARRAY_FIRST(&status_prompt_history));
+		ARRAY_REMOVE(&status_prompt_history, 0);
 	}
 
-	ARRAY_ADD(&c->prompt_hdata, xstrdup(c->prompt_buffer));
+	ARRAY_ADD(&status_prompt_history, xstrdup(line));
 }
 
 /* Complete word. */
