@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.134 2010/07/17 14:38:13 tcunha Exp $ */
+/* $Id: window.c,v 1.142 2010/12/06 22:52:21 nicm Exp $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,7 +24,6 @@
 #include <fnmatch.h>
 #include <pwd.h>
 #include <signal.h>
-#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <termios.h>
@@ -323,6 +322,9 @@ window_resize(struct window *w, u_int sx, u_int sy)
 void
 window_set_active_pane(struct window *w, struct window_pane *wp)
 {
+	if (wp == w->active)
+		return;
+	w->last = w->active;
 	w->active = wp;
 	while (!window_pane_visible(w->active)) {
 		w->active = TAILQ_PREV(w->active, window_panes, entry);
@@ -339,7 +341,7 @@ window_set_active_at(struct window *w, u_int x, u_int y)
 	struct window_pane	*wp;
 
 	TAILQ_FOREACH(wp, &w->panes, entry) {
-		if (!window_pane_visible(wp))
+		if (wp == w->active || !window_pane_visible(wp))
 			continue;
 		if (x < wp->xoff || x >= wp->xoff + wp->sx)
 			continue;
@@ -366,9 +368,16 @@ window_add_pane(struct window *w, u_int hlimit)
 void
 window_remove_pane(struct window *w, struct window_pane *wp)
 {
-	w->active = TAILQ_PREV(wp, window_panes, entry);
-	if (w->active == NULL)
-		w->active = TAILQ_NEXT(wp, entry);
+	if (wp == w->active) {
+		w->active = w->last;
+		w->last = NULL;
+		if (w->active == NULL) {
+			w->active = TAILQ_PREV(wp, window_panes, entry);
+			if (w->active == NULL)
+				w->active = TAILQ_NEXT(wp, entry);
+		}
+	} else if (wp == w->last)
+		w->last = NULL;
 
 	TAILQ_REMOVE(&w->panes, wp, entry);
 	window_pane_destroy(wp);
@@ -493,6 +502,8 @@ window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 void
 window_pane_destroy(struct window_pane *wp)
 {
+	window_pane_reset_mode(wp);
+
 	if (wp->fd != -1) {
 		close(wp->fd);
 		bufferevent_free(wp->event);
@@ -500,7 +511,6 @@ window_pane_destroy(struct window_pane *wp)
 
 	input_free(wp);
 
-	window_pane_reset_mode(wp);
 	screen_free(&wp->base);
 	if (wp->saved_grid != NULL)
 		grid_destroy(wp->saved_grid);
@@ -570,9 +580,11 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 		if (tcsetattr(STDIN_FILENO, TCSANOW, &tio2) != 0)
 			fatal("tcgetattr failed");
 
+		closefrom(STDERR_FILENO + 1);
+
 		environ_push(env);
 
-		clear_signals();
+		clear_signals(1);
 		log_close();
 
 		if (*wp->cmd != '\0') {
@@ -600,8 +612,6 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 		fatal("fcntl failed");
 	if (fcntl(wp->fd, F_SETFL, mode|O_NONBLOCK) == -1)
 		fatal("fcntl failed");
-	if (fcntl(wp->fd, F_SETFD, FD_CLOEXEC) == -1)
-		fatal("fcntl failed");
 	wp->event = bufferevent_new(wp->fd,
 	    window_pane_read_callback, NULL, window_pane_error_callback, wp);
 	bufferevent_enable(wp->event, EV_READ|EV_WRITE);
@@ -626,6 +636,14 @@ window_pane_read_callback(unused struct bufferevent *bufev, void *data)
 	input_parse(wp);
 
 	wp->pipe_off = EVBUFFER_LENGTH(wp->event->input);
+
+	/*
+	 * If we get here, we're not outputting anymore, so set the silence
+	 * flag on the window.
+	 */
+	wp->window->flags |= WINDOW_SILENCE;
+	if (gettimeofday(&wp->window->silence_timer, NULL) != 0)
+		fatal("gettimeofday failed.");
 }
 
 /* ARGSUSED */
